import math

from django import template
from django.utils.safestring import mark_safe
from history.utils import history
from main.templatetags.main_extras import pmrvalue, remix

register = template.Library()


@register.inclusion_tag("history/plot.html")
def history_plot(transactions, **kwargs):
    options = kwargs
    if "category" in kwargs or "account" in kwargs:
        options["history"] = history(transactions.all())
    else:
        options["history"] = history(transactions.exclude(category__budget=False))

    return options


@register.simple_tag
def calendar_opacity(v, vmax):
    if v is None:
        return "0%"
    return f"{math.sin(min(1, math.fabs(v/vmax))*math.pi/2):.0%}"


@register.simple_tag
def empty_calendar_cells(n):
    return mark_safe(n * "<td></td>")


@register.simple_tag
def empty_calendar_cells_start(n):
    return empty_calendar_cells(n - 1)


@register.simple_tag
def empty_calendar_cells_end(n):
    return empty_calendar_cells(12 - n)


@register.simple_tag
def up_down_icon(val):
    if val is None:
        return ""
    if val > 0:
        return remix("arrow-up-s", "green")
    elif val < 0:
        return remix("arrow-down-s", "red")

    return remix("equal", "white")


@register.simple_tag
def plot_bar(s, sum_pm, s_max):
    _res = ""

    if s_max:
        if sum_pm:
            _w = abs(sum_pm / s_max)
            _res += f"""<div style="width: {_w:.1%}"></div>"""
        if sum_pm is not None and s * sum_pm > 0:
            _w = abs(s / s_max)
            _res += (
                f"""<div class="tot" style="width: {_w:.1%}">"""
                f"""<span>{pmrvalue(s)}</span></div>"""
            )
    else:
        _res += "<div></div>"

    return mark_safe(_res)


@register.simple_tag
def calendar_head():
    months = range(1, 13)
    th = (f"""<th>{month:02d}</th>""" for month in months)

    return mark_safe("".join(th))


@register.filter
def sum_year(y_data):
    return sum(y["sum"] or 0 for y in y_data)
